////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	c3DGameView.h
// Desc:	A simple 3D viewer and interaction manager for the main game window
////////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Include directives

#define INITGUID
#include "antivirus.h"


//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data definitions

// w/h for the 3D viewer
sInt32 g3DVWidth = 800, 
		g3DVHeight = 600;

sInt32 gFontHeight = 14;


//-----------------------------------------------------------------------------
// Static data definitions

//-----------------------------------------------------------------------------
// Forward declaration of functions

//-----------------------------------------------------------------------------
// Function definitions

////////////////////////////////////////////////////////////////////////////////
// CALLBACK viewerProc
//
// default window proc for our D3D viewport window

LRESULT CALLBACK viewerProc( 
	HWND hWnd,				// owner window
	UINT msg,				// incoming message code
	WPARAM wParam,			// generic input parameter 1
	LPARAM lParam)			// generic input parameter 2
{
	switch (msg)
	{
		case WM_ACTIVATE:
			{
				RECT r;
				sInt32 rW, rH;

				// if we are being activated, try reaquiring DI devices
				if (wParam != WA_INACTIVE)
				{
					// get gameview ptr from userdata block, stored there in c3DGameView::init
					c3DGameView *gv = (c3DGameView*)GetWindowLong(hWnd, GWL_USERDATA);
					if (gv)
					{
						gv->getGameDIO()->reaquireDevices();
					}

					GetWindowRect(hWnd, &r);
					rW = (r.right - r.left)/2;
					rH = (r.bottom - r.top)/2;

					r.left += rW;
					r.top += rH;
					r.bottom -= rH;
					r.right -= rW;
					ClipCursor(&r);
					SetCursorPos(rW, rH);
					ShowCursor(FALSE);
				}
				else
				{
					ClipCursor(NULL);
					ShowCursor(TRUE);
				}
			}
			break;

		case WM_KEYUP:
			if (wParam == 27) // ESC key
				PostQuitMessage(1);
			break;

		case WM_CLOSE:
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
	}
	return DefWindowProc( hWnd, msg, wParam, lParam );
}


////////////////////////////////////////////////////////////////////////////////
// METHOD init
//
// initialise the class to defaults, ready for use

void c3DGameView::init(
	HINSTANCE hInst)			// application instance
{
	HRESULT hr;
	D3DDISPLAYMODE dmPrimary;
	LOGFONT rFontDesc;

	// save app handle
	mAppInstance = hInst;

	// create the Direct3D object
    mD3D = Direct3DCreate8( D3D_SDK_VERSION );
	assertWithMsg((mD3D != NULL), "DirectX 8 - Direct3D Could not be initialised (Direct3DCreate8)\n\nDo you have DirectX 8.1 installed?");

    // register the viewport windows class
    WNDCLASSEX wndClass = 
	{ 
		sizeof(WNDCLASSEX), 
		CS_CLASSDC, 
		viewerProc, 
		0, 
		0, 
		hInst,
        LoadIcon(hInst, MAKEINTRESOURCE(IDI_AV)),
        LoadCursor( NULL, IDC_ARROW ),
        NULL, 
		NULL,
		VIEWPORT_RENDER_WND_CLASS,
		NULL
	};
    RegisterClassEx( &wndClass );

    // create the render window
    mHWRender = CreateWindow(
		VIEWPORT_RENDER_WND_CLASS,
		"Antivirus - Press ESC to quit",
		WS_BORDER|WS_CLIPCHILDREN|WS_OVERLAPPEDWINDOW|WS_CAPTION,
		0, 0,
        0, 0,
		GetDesktopWindow(),
        0,
        hInst,
		0);

	// check to see if our window appears
	assertWithMsg((mHWRender != NULL), "Could not create viewport window");

	// center and show the viewport
	SetWindowPos(
		mHWRender,
		HWND_TOP,
		(GetSystemMetrics(SM_CXSCREEN) >> 1) - (g3DVWidth >> 1),
		(GetSystemMetrics(SM_CYSCREEN) >> 1) - (g3DVHeight >> 1),
		g3DVWidth,
		g3DVHeight,
		SWP_SHOWWINDOW);

	// get primary adapter desktop display format
	// for windowed 3d rendering (must match desktop format)
	mD3D->GetAdapterDisplayMode( 
		D3DADAPTER_DEFAULT, 
		&dmPrimary );

	// TODO: enumeration or checking of valid depth stencil formats
    // set up the presentation parameters
    ZeroMemory( &mParams, sizeof(D3DPRESENT_PARAMETERS) );
    mParams.Windowed				= TRUE;
    mParams.hDeviceWindow			= NULL;
	mParams.BackBufferFormat		= dmPrimary.Format;
    mParams.BackBufferCount			= 1;
	mParams.BackBufferHeight		= 0;
	mParams.BackBufferWidth			= 0;
    mParams.MultiSampleType			= D3DMULTISAMPLE_NONE;
    mParams.SwapEffect				= D3DSWAPEFFECT_COPY;
    mParams.EnableAutoDepthStencil	= TRUE;
	mParams.AutoDepthStencilFormat	= D3DFMT_D16;

	// create the rendering device
    hr = mD3D->CreateDevice( 
		D3DADAPTER_DEFAULT, 
		D3DDEVTYPE_HAL,
		mHWRender, 
		D3DCREATE_MIXED_VERTEXPROCESSING, 
		&mParams,
        &mDevice );

	// bail out
	if( FAILED(hr) )
	{
		assertWithMsg(FALSE, "Could not create viewport device on c3DGameView::init()");
	}

	// store pointer to main class in viewport window
	SetWindowLong(mHWRender, GWL_USERDATA, (LPARAM)this);

    // store backbuffer descriptor
    LPDIRECT3DSURFACE8 pBackBuffer;
    mDevice->GetBackBuffer( 0, D3DBACKBUFFER_TYPE_MONO, &pBackBuffer );
    pBackBuffer->GetDesc( &mBackBufferDesc );
    pBackBuffer->Release();

	// save default aspect
	mAspect = mBackBufferDesc.Width / (sFloat32)mBackBufferDesc.Height;


	// create a font renderer to use for general HUD / debugging
	GetObject(GetStockObject(SYSTEM_FONT), sizeof(LOGFONT), &rFontDesc);
	rFontDesc.lfHeight = (LONG)gFontHeight;
	rFontDesc.lfWidth = 0;
	strcpy((sChar8*)&rFontDesc.lfFaceName, "Arial");

	D3DXCreateFontIndirect(
		mDevice,
		&rFontDesc,
		&mFont);

	assertWithMsg( (mFont != NULL), "Could not create font on c3DGameView::init()");

	// set to default render states
	defaultStates();

	// fire up DirectInput
	mGameDIO = new cGameDIO();
	assertWithMsg( (mGameDIO != NULL), "Could not create instance of cGameDIO DI controller in c3DGameView::init()");
	mGameDIO->init(this, hInst);

	// setup 3D render mgr
	m3DRender = new c3DRender();
	assertWithMsg( (m3DRender != NULL), "m3DRender could not be created in c3DGameView::init()");
	m3DRender->init();
}

////////////////////////////////////////////////////////////////////////////////
// PROTECTED preRender
//
// prepare to render the scene

void c3DGameView::preRender(void)
{
	// clear the viewport
	mDevice->Clear( 0L, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0x00404040, 1.0f, 0L );

	// prepare device to render items
	mDevice->BeginScene();
}

////////////////////////////////////////////////////////////////////////////////
// PROTECTED postRender
//
// finish rendering and present result to screen

void c3DGameView::postRender(void)
{
	// finish rendering items
	mDevice->EndScene();

	// display render result
	mDevice->Present( NULL, NULL, NULL, NULL );
}

////////////////////////////////////////////////////////////////////////////////
// PROTECTED defaultStates
//
// sets the 3D device to its default render states

void c3DGameView::defaultStates(void)
{
    mDevice->SetRenderState( D3DRS_DITHERENABLE,		TRUE );
    mDevice->SetRenderState( D3DRS_ZENABLE,				TRUE );
    mDevice->SetRenderState( D3DRS_LIGHTING,			TRUE );

    // set up the default texture states
    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );
    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG1 );
    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_MINFILTER, D3DTEXF_LINEAR );
    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR );
    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_ADDRESSU,  D3DTADDRESS_WRAP );
    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_ADDRESSV,  D3DTADDRESS_WRAP );

	// enable alpha blending on textures
    gAVRTContext.m3DGameView->getDevice()->SetRenderState( D3DRS_ALPHABLENDENABLE,	TRUE );
    gAVRTContext.m3DGameView->getDevice()->SetRenderState( D3DRS_SRCBLEND,			D3DBLEND_SRCALPHA );
    gAVRTContext.m3DGameView->getDevice()->SetRenderState( D3DRS_DESTBLEND,			D3DBLEND_INVSRCALPHA );
}


////////////////////////////////////////////////////////////////////////////////
// PUBLIC render
//
// render something to the screen via app defined callback

void c3DGameView::render(
	renderCallback renderFn)
{
	// no device yet, just return
	if(!mDevice) return;

	// start
	preRender();

	// draw shit
	if(renderFn) 
	{
		// call rendering fn
		(renderFn)(this);
	}

	// stop
	postRender();
}

////////////////////////////////////////////////////////////////////////////////
// PUBLIC update
//
// ask the cGameDIO manager to update the controller data

void c3DGameView::update(void)
{
	// pass thru to gDIO update fn
	if (mGameDIO)
	{
		mGameDIO->update();
	}
}

////////////////////////////////////////////////////////////////////////////////
// METHOD tidyup
//
// releases any memory, returns to defaults, shutsdown the class after use

void c3DGameView::tidyup(void)
{
	if (mGameDIO)
	{
		mGameDIO->tidyup();
		delete mGameDIO;
	}

	SAFE_RELEASE(mFont);
	SAFE_RELEASE(mDevice);
	SAFE_RELEASE(mD3D);
}

////////////////////////////////////////////////////////////////////////////////
// METHOD c3DGameView
//
// constructor : nullify any class memory, reset to defaults

c3DGameView::c3DGameView(void)
{
	memset(this, 0, sizeof(c3DGameView));
}


////////////////////////////////////////////////////////////////////////////////
// METHODS DirectInput accessors
//
// used to check the current and last known states of various devices

sBool c3DGameView::isKbdButtonDown( sUInt32 dikcode )
{
	return (mGameDIO->mCurFrameData.mKeyboard[dikcode]&0x80);
}

sBool c3DGameView::isKbdButtonJustPressed( sUInt32 dikcode )
{
	if( (mGameDIO->mCurFrameData.mKeyboard[dikcode]&0x80) &&
		!(mGameDIO->mLastFrameData.mKeyboard[dikcode]&0x80)) return TRUE;
	return FALSE;
}

sBool c3DGameView::isMouseButtonDown( sUInt8 btnNo )
{
	return (mGameDIO->mCurFrameData.mMouse.rgbButtons[btnNo]&0x80);
}

sBool c3DGameView::isMouseButtonJustPressed( sUInt8 btnNo )
{
	if( (mGameDIO->mCurFrameData.mMouse.rgbButtons[btnNo]&0x80) &&
		!(mGameDIO->mLastFrameData.mMouse.rgbButtons[btnNo]&0x80)) return TRUE;
	return FALSE;
}



////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////
// PUBLIC init
//
// initialise the class to defaults, ready for use

void cGameDIO::init(
	c3DGameView *owner,					// owner c3DGameView class
	HINSTANCE hInst)					// application instance handle
{
	HRESULT hr;

	// keep tabs on owning game view
	mGameViewHost = owner;

	// connect to DirectInput
	hr = DirectInput8Create(
		hInst,
		DIRECTINPUT_VERSION,
		IID_IDirectInput8,
		(void**)&mDIO,
		NULL);

	// check for a valid connection
	assertWithMsg( (mDIO != NULL), "Could not connect to DirectInput8");

	// attach the devices we require
	aquireDevices();
}

////////////////////////////////////////////////////////////////////////////////
// PUBLIC aquireDevices
//
// get connections to the devices to get data from

void cGameDIO::aquireDevices(void)
{
	HRESULT hr;

	// keyboard and mouse GUIDs are predefined by the system, which is nice
	hr = mDIO->CreateDevice(GUID_SysKeyboard, &mDevKeyboard, NULL);
	hr = mDIO->CreateDevice(GUID_SysMouse, &mDevMouse, NULL);

	// assign cooperation levels to the keyboard and mouse
	hr = mDevKeyboard->SetCooperativeLevel(mGameViewHost->mHWRender, 
				DISCL_NONEXCLUSIVE | DISCL_FOREGROUND);
	hr = mDevMouse->SetCooperativeLevel(mGameViewHost->mHWRender, 
				DISCL_NONEXCLUSIVE | DISCL_FOREGROUND);

	// set data formats
	mDevKeyboard->SetDataFormat(&c_dfDIKeyboard);
	mDevMouse->SetDataFormat(&c_dfDIMouse);


	// aquire device interfaces
	mDevKeyboard->Acquire();
	mDevMouse->Acquire();
}

////////////////////////////////////////////////////////////////////////////////
// PUBLIC reaquireDevices
//
// connect back to devices if connections are lost

void cGameDIO::reaquireDevices(void)
{
	if(mDevKeyboard)	mDevKeyboard->Acquire();
	if(mDevMouse)		mDevMouse->Acquire();
}

////////////////////////////////////////////////////////////////////////////////
// PUBLIC releaseDevices
//
// release any connections to bound devices

void cGameDIO::releaseDevices(void)
{
	// release interfaces, just in case
	if(mDevKeyboard)	mDevKeyboard->Unacquire();
	if(mDevMouse)		mDevMouse->Unacquire();

	// kill pointers
	SAFE_RELEASE(mDevKeyboard);
	SAFE_RELEASE(mDevMouse);
}

////////////////////////////////////////////////////////////////////////////////
// PUBLIC update
//
// gets the latest data from the connected controllers

void cGameDIO::update(void)
{
	// save last frame data
	memcpy(&mLastFrameData, &mCurFrameData, sizeof(sIOData));

	// get latest data
	mDevKeyboard->GetDeviceState(sizeof(sChar8)*256,mCurFrameData.mKeyboard);
	mDevMouse->GetDeviceState(sizeof(DIMOUSESTATE),&mCurFrameData.mMouse);
}

////////////////////////////////////////////////////////////////////////////////
// METHOD tidyup
//
// releases any memory, returns to defaults, shutsdown the class after use

void cGameDIO::tidyup(void)
{
	releaseDevices();

	SAFE_RELEASE(mDIO);
}
