////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	c3DGameView.h
// Desc:	A simple 3D viewer and interaction manager for the main game window
////////////////////////////////////////////////////////////////////////////////

#ifndef HDD_ANTIVIRUS_CORE_C3DGAMEVIEW_H
#define HDD_ANTIVIRUS_CORE_C3DGAMEVIEW_H


//-----------------------------------------------------------------------------
// Include directives

//-----------------------------------------------------------------------------
// Macro definitions

#define VIEWPORT_RENDER_WND_CLASS		("ANTIVIRUS_3DVIEW_WND")


//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

class c3DGameView;

// render callback function pointer
typedef void (*renderCallback)(c3DGameView *gview);

////////////////////////////////////////////////////////////////////////////////
// STRUCT cIOData
//
// stores a single captured frame of controller data via DirectInput

typedef struct sIOData
{
	char					mKeyboard[256];
	DIMOUSESTATE			mMouse;

} sIOData;


////////////////////////////////////////////////////////////////////////////////
// CLASS cGameDIO
//
// IO control for the game window
// uses DirectInput8 to get data from keyboard and mouse efficiently

class cGameDIO
{
private:
	//-------------------------------------------------------------
	// Private data

	c3DGameView				*mGameViewHost;	// ptr to owning c3DGameView class

public:
	//-------------------------------------------------------------
	// Public data

	IDirectInput8			*mDIO;			// DirectInput used for controller reads

	LPDIRECTINPUTDEVICE8	mDevKeyboard;
	LPDIRECTINPUTDEVICE8	mDevMouse;

	sIOData					mCurFrameData, 
							mLastFrameData;


	//-------------------------------------------------------------
	// Public functions

	////////////////////////////////////////////////////////////////////////////////
	// PUBLIC aquireDevices
	//
	// get connections to the devices to get data from

	void aquireDevices(void);

	////////////////////////////////////////////////////////////////////////////////
	// PUBLIC reaquireDevices
	//
	// connect back to devices if connections are lost

	void reaquireDevices(void);

	////////////////////////////////////////////////////////////////////////////////
	// PUBLIC releaseDevices
	//
	// release any connections to bound devices

	void releaseDevices(void);


	////////////////////////////////////////////////////////////////////////////////
	// PUBLIC update
	//
	// gets the latest data from the connected controllers

	void update(void);


	////////////////////////////////////////////////////////////////////////////////
	// PUBLIC init
	//
	// initialise the class to defaults, ready for use

	void init(
		c3DGameView *owner,					// owner c3DGameView class
		HINSTANCE hInst);					// application instance handle

	////////////////////////////////////////////////////////////////////////////////
	// METHOD tidyup
	//
	// releases any memory, returns to defaults, shutsdown the class after use

	void tidyup(void);


	// constructor
	cGameDIO (void)
	{
		memset(this, 0, sizeof(cGameDIO));
	}
};

////////////////////////////////////////////////////////////////////////////////
// CLASS c3DGameView
//
// the 3D game view controls all DirectX end functionality, providing a
// rendering context and DirectInput support for mouse and keyboard

class c3DGameView
{
friend cGameDIO;
private:
	//-------------------------------------------------------------
	// Private data

	HINSTANCE				mAppInstance;		// saved app instance

protected:
	//-------------------------------------------------------------
	// Protected data

	IDirect3D8				*mD3D;			// instance of DX8 interface
	cGameDIO				*mGameDIO;		// game controller input capturing

    IDirect3DDevice8		*mDevice;		// active viewport
	ID3DXFont				*mFont;			// font used to write on viewport

	D3DPRESENT_PARAMETERS	mParams;		// presentation params used
	D3DSURFACE_DESC			mBackBufferDesc;// active backbuffer description
	
	sFloat32				mAspect;		// aspect ratio calculated at creation time

	HWND					mHWRender;		// rendering window handle


	//-------------------------------------------------------------
	// Protected functions

	////////////////////////////////////////////////////////////////////////////////
	// PROTECTED preRender
	//
	// prepare to render the scene

	void preRender(void);

	////////////////////////////////////////////////////////////////////////////////
	// PROTECTED postRender
	//
	// finish rendering and present result to screen

	void postRender(void);

	////////////////////////////////////////////////////////////////////////////////
	// PROTECTED defaultStates
	//
	// sets the 3D device to its default render states

	void defaultStates(void);


public:
	//-------------------------------------------------------------
	// Public data

	// 3d rendering setup
	c3DRender				*m3DRender;


	//-------------------------------------------------------------
	// Public functions

	////////////////////////////////////////////////////////////////////////////////
	// METHOD init
	//
	// initialise the class to defaults, ready for use

	void init(
		HINSTANCE hInst);			// application instance

	////////////////////////////////////////////////////////////////////////////////
	// PUBLIC render
	//
	// render something to the screen via app defined callback

	void render(
		renderCallback renderFn);

	////////////////////////////////////////////////////////////////////////////////
	// PUBLIC update
	//
	// ask the cGameDIO manager to update the controller data

	void update(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD tidyup
	//
	// releases any memory, returns to defaults, shutsdown the class after use

	void tidyup(void);

	

	// constructor
	c3DGameView (void);


	// Accessors
	IDirect3D8			*getD3DInterface() { return mD3D; }
    IDirect3DDevice8	*getDevice() { return mDevice; }
	ID3DXFont			*getFont() { return mFont; }
	sFloat32			getAspect() { return mAspect; }
	cGameDIO			*getGameDIO() { return mGameDIO; }
	HWND				getHWND() { return mHWRender; }

	sInt32				getWidth() { return mBackBufferDesc.Width; }
	sInt32				getHeight() { return mBackBufferDesc.Height; }

	// DI device Accessors
	sBool				isKbdButtonDown( sUInt32 dikcode );
	sBool				isKbdButtonJustPressed( sUInt32 dikcode );
	sBool				isMouseButtonDown( sUInt8 btnNo );
	sBool				isMouseButtonJustPressed( sUInt8 btnNo );
};


//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data declarations

// single instance of the 3D game viewer class
extern c3DGameView		gGameView;

extern sInt32 g3DVWidth, 
			g3DVHeight;

extern sInt32 gFontHeight;

//------------------------------------------------------------------------------
// Global function declarations



#endif // HDD_ANTIVIRUS_CORE_C3DGAMEVIEW_H
