////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cCureChain.h
// Desc:	Similar to the virus chain, this supplies the user-laid cure blocks out
////////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Include directives

#include "antivirus.h"

//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data definitions

//-----------------------------------------------------------------------------
// Static data definitions

//-----------------------------------------------------------------------------
// Forward declaration of functions

//-----------------------------------------------------------------------------
// Function definitions

////////////////////////////////////////////////////////////////////////////////
// METHOD create
//
// create a new sequence from the active cure sequence. this is called
// by the gameplay manager

void cCureChain::create(
	sInt32 lat,
	sFloat32 ypos,
	cCureSeq *cs)
{
	// copy sequence
	mNumViralCodes = cs->mNumViralCodes;
	memcpy(mViralCodes, cs->mViralCodes, sizeof(eViralCode) * mNumViralCodes);

	// set life timer (seconds)
	mLife = 6.0f;

	mLateralIdx = lat;
	mYPos = ypos;

	// activate cure chain
	mActive = TRUE;

	mIsDying = FALSE;
	mDyingEmissive = 0;

	mYStart = ypos + 0.1f;
	mYEnd = ypos - 0.3f;
}

////////////////////////////////////////////////////////////////////////////////
// METHOD update
//
// called via game loop, once per frame. updates class for the current frame.

void cCureChain::update(void)
{
	if (!mActive) return;

	// form tm matrix
	D3DXMatrixTranslation(&mTM, (sFloat32)mLateralIdx, mYPos, 0.0f);

	// update life timer
	mLife -= gFrameTime;
	if (mLife <= 0.0f)
	{
		mActive = FALSE;
	}

	if (mIsDying)
	{
		mDyingEmissive += 1.0f * (gFrameTime * 3.0f);
		CLAMP_HIGH(mDyingEmissive, 1.0f);

		if (mDyingEmissive >= 1.0f)
		{
			mActive = FALSE;
			mIsDying = FALSE;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// METHOD draw
//
// renders any graphics associated with this class (HUD, debug, etc)

void cCureChain::draw(
	c3DGameView *gview)
{
	sInt32 i;

	if (!mActive) return;

	// draw the chain
	for (i=0; i<mNumViralCodes; i++)
	{
		// render mesh
		gview->getDevice()->SetTransform(D3DTS_WORLD, &mTM);
		gAVRTContext.mGameState->mCureData->renderCureBlock(
			(mLife<1.0f)?mLife:1.0f,
			mViralCodes[i],
			this);

		// next slot in X
		mTM.m[3][0] += 1.0f;
	}
}

////////////////////////////////////////////////////////////////////////////////
// METHOD hasCollided
//
// return TRUE if this chain collides with the item defined by the inputs

sBool cCureChain::hasCollided(
	sInt32 lat,
	sInt32 size,
	sFloat32 ypos,
	sInt32 &colIdx)
{
	sInt32 latDelt = lat - mLateralIdx;
	sInt32 fullLat = (mLateralIdx + mNumViralCodes);

	if (ypos <= mYStart && ypos >= mYEnd)
	{
		if (latDelt <= 0 && latDelt > -size)
		{
			colIdx = latDelt;
			return TRUE;
		}

		if (
			(lat >= mLateralIdx && lat <= fullLat) && (lat + size) <= fullLat)
		{
			colIdx = -1;
			return TRUE;
		}
	}

	return FALSE;
}


////////////////////////////////////////////////////////////////////////////////
// METHOD viralMatch
//
// does this cure chain link with the given viral chain?

sBool cCureChain::viralMatch(
	cViralChain *vc)
{
	sInt32 i;

	// code count doesn't match up, bail out
	if (vc->mNumViralCodes != mNumViralCodes)
	{
		return FALSE;
	}

	// match up the codes
	for (i=0; i<vc->mNumViralCodes; i++)
	{
		if (mViralCodes[i] != vc->mViralCodes[i])
		{
			return FALSE;
		}
	}
	return TRUE;
}


////////////////////////////////////////////////////////////////////////////////
// METHOD cCureChain
//
// constructor : nullify any class memory, reset to defaults

cCureChain::cCureChain(void)
{
	memset(this, 0, sizeof(cViralChain));
}

