////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cGameState.h
// Desc:	Controls the flow of game logic
////////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Include directives

#include "antivirus.h"

//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data definitions

// text descriptions for difficulty modes
sChar8	*gDifficultyName[NUM_GAME_DIFF_SETTINGS] =
{
	"Bacterial Banisher",
	"Viral Vanquisher",
	"DNA Dominator",
	"Chemical Conqueror",
};

sChar8	*gDifficultyDesc[NUM_GAME_DIFF_SETTINGS] = 
{
	"Slow virii, limited colours, simple sequences. For beginners!",
	"Some more complex sequences, keep your eyes peeled!",
	"Virii cascading down, plenty of colours and taxing sequences.",
	"A veritable plague of virii with some mad sequences. For the hardcore players!"
};


//-----------------------------------------------------------------------------
// Static data definitions

//-----------------------------------------------------------------------------
// Forward declaration of functions

//-----------------------------------------------------------------------------
// Function definitions

////////////////////////////////////////////////////////////////////////////////
// METHOD init
//
// initialise the class to defaults, ready for use

void cGameState::init(void)
{
	// default to showing the intro gui
	setGameMode(GM_AV_GUI);

	// default difficulty
	setGameDiff(GD_AV_NORMAL);

	// setup mouse pointer manager
	mMousePtr = new cMousePtr();
	assertWithMsg( (mMousePtr != NULL), "mMousePtr could not be created in cGameState::init()");
	mMousePtr->init();

	// load viral meshes / mtl data
	mViralData = new cViralData();
	assertWithMsg( (mViralData != NULL), "mViralData could not be created in cGameState::init()");
	mViralData->init();

	// load cure mesh
	mCureData = new cCureData();
	assertWithMsg( (mCureData != NULL), "mCureData could not be created in cGameState::init()");
	mCureData->init();

	// new hiscores manager
	mHiscores = new cHiscores();
	assertWithMsg( (mHiscores != NULL), "mHiscores could not be created in cGameState::init()");
	mHiscores->init();

	// the gameplay HUD
	mHUD = new cHUD();
	assertWithMsg( (mHUD != NULL), "mHUD could not be created in cGameState::init()");
	mHUD->init();

	// start up the gameplay manager
	mGameplay = new cGameplay();
	assertWithMsg( (mGameplay != NULL), "mGameplay could not be created in cGameState::init()");
	mGameplay->init();

	// new GUI manager
	mGUI = new cGUI();
	assertWithMsg( (mGUI != NULL), "mGUI could not be created in cGameState::init()");
	mGUI->init();
}

////////////////////////////////////////////////////////////////////////////////
// METHOD update
//
// called via game loop, once per frame. updates class for the current frame.

void cGameState::update(void)
{
	// always update mouse pointer
	mMousePtr->update();

	switch (mGameMode)
	{
		// intro GUI
		case GM_AV_GUI:
			{
				mGUI->update();
			}
			break;

		// main game mode
		case GM_AV_PLAY:
			{
				mGameplay->update();
				mHUD->update();
			}
			break;

		// game has ended
		case GM_AV_GAMEOVER:
			{
			}
			break;

		// show hiscore table
		case GM_AV_HISCORES:
			{
				mHiscores->update();
			}
			break;
	}
}

////////////////////////////////////////////////////////////////////////////////
// METHOD draw
//
// renders any graphics associated with this class (HUD, debug, etc)

void cGameState::draw(
	c3DGameView *gview)
{
	switch (mGameMode)
	{
		// intro GUI
		case GM_AV_GUI:
			{
				mGUI->draw(gview);
			}
			break;

		// main game mode
		case GM_AV_PLAY:
			{
				mGameplay->draw(gview);
				mHUD->draw(gview);
			}
			break;

		// game has ended
		case GM_AV_GAMEOVER:
			{
			}
			break;

		// show hiscore table
		case GM_AV_HISCORES:
			{
				mHiscores->draw(gview);
			}
			break;
	}

	// always draw mouse pointer
	mMousePtr->draw();
}

////////////////////////////////////////////////////////////////////////////////
// METHOD tidyup
//
// releases any memory, returns to defaults, shutsdown the class after use

void cGameState::tidyup(void)
{
	mGUI->tidyup();
	delete mGUI;
}

////////////////////////////////////////////////////////////////////////////////
// METHOD cGameState
//
// constructor : nullify any class memory, reset to defaults

cGameState::cGameState(void)
{
	memset(this, 0, sizeof(cGameState));
}


////////////////////////////////////////////////////////////////////////////////
// METHOD setGameMode
//
// set a new game mode, resetting the incoming mode

void cGameState::setGameMode(eGameMode gm)
{
	switch (mGameMode)
	{
		// intro GUI
		case GM_AV_GUI:
			{
			}
			break;

		// main game mode
		case GM_AV_PLAY:
			{
				mHUD->reset();
				mGameplay->reset();
			}
			break;

		// game has ended
		case GM_AV_GAMEOVER:
			{
			}
			break;

		// show hiscore table
		case GM_AV_HISCORES:
			{
				mHiscores->reset();
			}
			break;
	}

	mGameMode = gm;
}


