////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cGameplay.h
// Desc:	Code and managers running the main show - the antivirus game code
////////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Include directives

#include "antivirus.h"


//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data definitions

////////////////////////////////////////////////////////////////////////////////
// difficulty definitions

// health starting points per difficulty level
sInt32 gGPMaxHealth[NUM_GAME_DIFF_SETTINGS] = 
{
	250,
	200,
	150,
	100
};

// min/max respawning time
sFloat32 gMinRespawn[NUM_GAME_DIFF_SETTINGS] = 
{
	2.5f,	1.5f,	1.25f,	0.75f,
};
sFloat32 gMaxRespawn[NUM_GAME_DIFF_SETTINGS] = 
{
	3.0f,	2.5f,	2.5f,	2.0f,
};

sInt32 gMaxViralLinks[NUM_GAME_DIFF_SETTINGS] = 
{
	3, 5, 6, gMaxNumViralCodeList,
};

//-----------------------------------------------------------------------------
// Static data definitions

//-----------------------------------------------------------------------------
// Forward declaration of functions

//-----------------------------------------------------------------------------
// Function definitions

////////////////////////////////////////////////////////////////////////////////
// METHOD init
//
// initialise the class to defaults, ready for use

void cGameplay::init(void)
{
	// initialise cure sequence
	mCureSeq = new cCureSeq();
	mCureSeq->init();

	// go to defauls
	reset();
}

////////////////////////////////////////////////////////////////////////////////
// METHOD reset
//
// reset the class back to default state

void cGameplay::reset(void)
{
	eGameDifficulty gDiff = gAVRTContext.mGameState->getGameDiff();

	// set health to defaults
	mMaxHealth = gGPMaxHealth[gDiff];
	mHealth = mMaxHealth;

	// reset score
	mCurScore = 0;

	// reset chain list
	memset(mViralChain, 0, sizeof(cViralChain) * gMaxActiveViralChains);
	mLastSpawnIdx = -1;

	mTimeTillNextSpawn = 0.0f;
}

////////////////////////////////////////////////////////////////////////////////
// METHOD update
//
// called via game loop, once per frame. updates class for the current frame.

void cGameplay::update(void)
{
	sInt32 i, j, rpower, newSz, colIdx, diffIdx = (sInt32)gAVRTContext.mGameState->getGameDiff();
	sBool spawnPowerup;

	// update active cure sequence
	mCureSeq->update(this);

	mTimeTillNextSpawn -= gFrameTime;

	// time to try and spawn another virii
	if (mTimeTillNextSpawn <= 0.0f)
	{
		for (i=0; i<gMaxActiveViralChains; i++)
		{
			if (!mViralChain[i].mActive)
			{
				// do we want a powerup
				rpower = (sInt32)RANDOM(100.0f);
				spawnPowerup = (rpower>90);

				newSz = (sInt32)RANDOM(gMaxViralLinks[diffIdx]);

				// if powerup, make sure there are at least 4 links
				if (spawnPowerup)
				{
					CLAMP_LOW(newSz, 4);
				}
				mViralChain[i].spawn(RANDOM(16-newSz), newSz, spawnPowerup);

				mLastSpawnIdx = i;
				break;
			}
		}

		mTimeTillNextSpawn = gMinRespawn[diffIdx] + RANDOM(gMinRespawn[diffIdx]);
	}

	// update all chains created
	for (i=0; i<gMaxActiveViralChains; i++)
	{
		if (mViralChain[i].hasInfected())
		{
			// whack health off
			mHealth -= (mViralChain[i].mNumViralCodes * 10);
			mViralChain[i].mActive = FALSE;

			// pulse blood flow
			gAVRTContext.mGameState->mHUD->mBloodPulse = 128;
		}
		
		mViralChain[i].update();
		mCureChain[i].update();
	}
	
	// virus -> cure collision loop
	// a bit recursive and ugly, could do with some smartening up and optimising
	for (i=0; i<gMaxActiveViralChains; i++)
	{
		for (j=0; j<gMaxActiveViralChains; j++)
		{
			if (mCureChain[i].mActive && !mCureChain[i].mIsDying)
			{
				if (mViralChain[j].mActive)
				{
					// check this curechain against active virus
					if (mCureChain[i].hasCollided(
						mViralChain[j].mLateralIdx, 
						mViralChain[j].mNumViralCodes, 
						mViralChain[j].mFallPos,
						colIdx))
					{
						// remove cure chain
						mCureChain[i].mIsDying = TRUE;

						// is this a end-to-end match?
						if (colIdx == 0)
						{
							// check the viral codes
							if (mCureChain[i].viralMatch(&mViralChain[j]))
							{
								// remove viral chain, success!
								mViralChain[j].mIsDying = TRUE;

								// calc and update score, and handle powerups
								if (mViralChain[j].mIsPowerup)
								{
									mHealth += 50;
									CLAMP_HIGH(mHealth, mMaxHealth);

									mCurScore += 
										(mViralChain[j].mNumViralCodes * 200) * ((diffIdx+1) * 2);
								}
								else
								{
									mCurScore += 
										(mViralChain[j].mNumViralCodes * 50) * ((diffIdx+1) * 2);
								}
							}
						}
					}
				}
			}
		}
	}

	// check for death
	if (mHealth <= 0)
	{
		// add my score to the table
		gAVRTContext.mGameState->mHiscores->addScore(mCurScore);
		gAVRTContext.mGameState->setGameMode(GM_AV_HISCORES);
	}
}

////////////////////////////////////////////////////////////////////////////////
// METHOD draw
//
// renders any graphics associated with this class (HUD, debug, etc)

void cGameplay::draw(
	c3DGameView *gview)
{
	sInt32 i;

	// get ready to render
	gview->m3DRender->applyViewMatrix(gview);

	// update active cure sequence
	mCureSeq->draw(gview);

	// draw all the cures
	for (i=0; i<gMaxActiveViralChains; i++)
	{
		mCureChain[i].draw(gview);
	}

	// draw all the chains
	for (i=0; i<gMaxActiveViralChains; i++)
	{
		mViralChain[i].draw(gview);
	}
}

////////////////////////////////////////////////////////////////////////////////
// METHOD tidyup
//
// releases any memory, returns to defaults, shutsdown the class after use

void cGameplay::tidyup(void)
{
}

////////////////////////////////////////////////////////////////////////////////
// METHOD cGameplay
//
// constructor : nullify any class memory, reset to defaults

cGameplay::cGameplay(void)
{
}

