////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cHiscores.h
// Desc:	Every game needs a hiscore table!
////////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Include directives

#include "antivirus.h"

//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data definitions

//-----------------------------------------------------------------------------
// Static data definitions

//-----------------------------------------------------------------------------
// Forward declaration of functions

//-----------------------------------------------------------------------------
// Function definitions

////////////////////////////////////////////////////////////////////////////////
// METHOD init
//
// initialise the class to defaults, ready for use

void cHiscores::init(void)
{
	// make default entries in the table
	genDefaults();

	// load any saved items
	load();

	reset();
}

////////////////////////////////////////////////////////////////////////////////
// METHOD reset
//
// reset the class back to default state

void cHiscores::reset(void)
{
	mAlphaFade = 0;
}

////////////////////////////////////////////////////////////////////////////////
// METHOD genDefaults
//
// build default scores, always called before loading official ones

void cHiscores::genDefaults(void)
{
	sInt32 i;
	LOGFONT rFontDesc;

	// just build some basic entries for people to beat!
	for (i=0; i<MAX_HIS_ENTRIES; i++)
	{
		mHSTable[i].mDiffLevel = GD_AV_NORMAL;
		strcpy(mHSTable[i].mName, "Ishani");
		mHSTable[i].mScore = (MAX_HIS_ENTRIES * 1500) - (i * 1000);
	}

	// create a larger font to use when drawing title
	GetObject(GetStockObject(SYSTEM_FONT), sizeof(LOGFONT), &rFontDesc);
	rFontDesc.lfHeight = 36;
	rFontDesc.lfWeight = 800;
	rFontDesc.lfWidth = 0;
	strcpy((sChar8*)&rFontDesc.lfFaceName, "Verdana");

	D3DXCreateFontIndirect(
		gAVRTContext.m3DGameView->getDevice(),
		&rFontDesc,
		&mFontTitle);
}

////////////////////////////////////////////////////////////////////////////////
// METHOD addScore
//
// adds the score with the current global player name to the chart

void cHiscores::addScore(
	sUInt32 score)
{
	sInt32 i, j;

	for (i=0; i<MAX_HIS_ENTRIES; i++)
	{
		if (score >= mHSTable[i].mScore)
		{
			for (j = (MAX_HIS_ENTRIES-2); j > i; j--)
			{
				memcpy(&mHSTable[j+1], &mHSTable[j], sizeof(sHiscoreEntry));
			}

			strcpy(mHSTable[i].mName, gPlayerName);
			mHSTable[i].mScore = score;
			mHSTable[i].mDiffLevel = gAVRTContext.mGameState->getGameDiff();

			return;
		}
	}

	// save changes to disk
	save();
}

////////////////////////////////////////////////////////////////////////////////
// METHOD load
//
// load saved scores, if they are available

void cHiscores::load(void)
{
	FILE *stream = fopen("avHIS.dat", "rb");
	if (stream)
	{
		fread(mHSTable, sizeof(sHiscoreEntry), MAX_HIS_ENTRIES, stream);
		fclose(stream);
	}
}

////////////////////////////////////////////////////////////////////////////////
// METHOD save
//
// save current scores

void cHiscores::save(void)
{
	FILE *stream = fopen("avHIS.dat", "wb");
	if (stream)
	{
		fwrite(mHSTable, sizeof(sHiscoreEntry), MAX_HIS_ENTRIES, stream);
		fclose(stream);
	}
}

////////////////////////////////////////////////////////////////////////////////
// METHOD update
//
// called via game loop, once per frame. updates class for the current frame.

void cHiscores::update(void)
{
	// update fade-in
	if (mAlphaFade < 1024)
	{
		mAlphaFade += 8;
	}

	// back to GUI on [Enter]
	if (gAVRTContext.m3DGameView->isKbdButtonJustPressed(DIK_RETURN))
	{
		gAVRTContext.mGameState->setGameMode(GM_AV_GUI);
	}
}

////////////////////////////////////////////////////////////////////////////////
// METHOD draw
//
// renders any graphics associated with this class (HUD, debug, etc)

void cHiscores::draw(
	c3DGameView *gview)
{
	RECT fontRect;
	sInt32 i, newAlpha;
	sChar8 hisOutput[255];

	// default position
	fontRect.left = (sInt32)(0.1f * (sFloat32)gview->getHeight());
	fontRect.top = (sInt32)(0.2f * (sFloat32)gview->getHeight());
	fontRect.right = gview->getWidth();
	fontRect.bottom = gview->getHeight();

	// switch to 2D mode
	c2DRender::applyViewMatrix(gview);

	// begin rendering
	gview->getFont()->Begin();

	// draw column header
	gview->getFont()->DrawText(
		"Score            Name - Difficulty",
		-1,
		&fontRect,
		DT_LEFT,
		D3DCOLOR_RGBA(255,192,0,255));
	fontRect.top += 20;

	// draw column data
	for (i=0; i<MAX_HIS_ENTRIES; i++)
	{
		sprintf(hisOutput,
			"%08i       %s - %s",
			mHSTable[i].mScore,
			mHSTable[i].mName,
			gDifficultyName[mHSTable[i].mDiffLevel]);

		newAlpha = mAlphaFade - (i * 64);
		CLAMP(newAlpha, 0, 255);

		gview->getFont()->DrawText(
			hisOutput,
			-1,
			&fontRect,
			DT_LEFT,
			D3DCOLOR_RGBA(255,255,255,newAlpha));

		fontRect.top += 20;
	}

	// finish rendering
	gview->getFont()->End();

	// look to top right
	fontRect.left = 0;
	fontRect.top = (sInt32)(0.05f * (sFloat32)gview->getHeight());
	fontRect.right = mAlphaFade << 1;
	CLAMP_HIGH(fontRect.right, gview->getWidth());

	// render title
	mFontTitle->Begin();

	newAlpha = mAlphaFade / 4;
	CLAMP(newAlpha, 0, 255);

	mFontTitle->DrawText(
		"Antivirus Top Scores    ",
		-1,
		&fontRect,
		DT_RIGHT,
		D3DCOLOR_RGBA(255,255,255,newAlpha));

	fontRect.left = 0;
	fontRect.top = (sInt32)(0.9f * (sFloat32)gview->getHeight());
	fontRect.right = gview->getWidth();

	mFontTitle->DrawText(
		"Press Enter To Continue",
		-1,
		&fontRect,
		DT_CENTER,
		D3DCOLOR_RGBA(255,255,255,128));

	mFontTitle->End();
}

////////////////////////////////////////////////////////////////////////////////
// METHOD tidyup
//
// releases any memory, returns to defaults, shutsdown the class after use

void cHiscores::tidyup(void)
{
	SAFE_RELEASE(mFontTitle);
}

////////////////////////////////////////////////////////////////////////////////
// METHOD cHiscores
//
// constructor : nullify any class memory, reset to defaults

cHiscores::cHiscores(void)
{
	memset(this, 0, sizeof(cHiscores));
}

