////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cViralChain.h
// Desc:	Management for a single viral strand
////////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Include directives

#include "antivirus.h"

//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data definitions

// the maximum choice of viral codes per difficultly level
sInt32 gMaxViralCodeChoices[NUM_GAME_DIFF_SETTINGS] =
{
	VC_YELLOW,
	VC_PURPLE,
	VC_WHITE,
	NUM_VIRAL_CODES
};

sFloat32 gViralFallSpeed[NUM_GAME_DIFF_SETTINGS] =
{
	1.0f,
	1.5f,
	2.0f,
	2.0f
};


//-----------------------------------------------------------------------------
// Static data definitions

//-----------------------------------------------------------------------------
// Forward declaration of functions

//-----------------------------------------------------------------------------
// Function definitions

////////////////////////////////////////////////////////////////////////////////
// METHOD spawn
//
// spawn a new viral chain, at the given point with the correct parameters

void cViralChain::spawn(
	sInt32 latIdx,				// x slot to spawn into
	sInt32 size,				// length of chain
	sBool powerup)				// is this a powerup
{
	sInt32 i;
	eGameDifficulty gDiff = gAVRTContext.mGameState->getGameDiff();

	// new slot pos
	mLateralIdx = latIdx;

	mFallPos = 1.0f;
	mFallSpeed = gViralFallSpeed[gDiff] * gFrameTime;

	mCurYRot = RANDOM(50.0f);
	mYRotSpeed = D3DXToRadian((45.0f * gViralFallSpeed[gDiff]) * gFrameTime);

	if ((sInt32)mCurYRot & 1)
	{
		mYRotSpeed = -mYRotSpeed;
	}

	// assign codes
	mNumViralCodes = size;
	CLAMP(mNumViralCodes, 1, gMaxNumViralCodeList);

	mIsPowerup = powerup;
	mIsDying = FALSE;
	mDyingEmissive = 0;

	// make new codes!
	for (i=0; i<mNumViralCodes; i++)
	{
		// new random code
		mViralCodes[i] = (eViralCode)(sInt32)RANDOM(gMaxViralCodeChoices[gDiff]);

		// new mesh variant
		if (mIsPowerup)
		{
			mCurMeshVariant[i] = gNumViralMeshes-1;
		}
		else
		{
			mCurMeshVariant[i] = (sInt32)RANDOM(gNumViralMeshes-1);
		}
	}

	mActive = TRUE;
}

////////////////////////////////////////////////////////////////////////////////
// METHOD update
//
// called via game loop, once per frame. updates class for the current frame.

void cViralChain::update(void)
{
	if (!mActive) return;

	// form spin matrix
	D3DXMatrixRotationX(&mTM, mCurYRot);

	// set translation
	mTM.m[3][0] = (sFloat32)(mLateralIdx);
	mTM.m[3][1] = (sFloat32)(mFallPos);

	// check on dying flag
	if (mIsDying)
	{
		mDyingEmissive += 1.0f * (gFrameTime * 3.0f);
		CLAMP_HIGH(mDyingEmissive, 1.0f);

		if (mDyingEmissive >= 1.0f)
		{
			mActive = FALSE;
			mIsDying = FALSE;
		}
	}
	else
	{
		mCurYRot += mYRotSpeed;
		mFallPos -= mFallSpeed;
	}
}

////////////////////////////////////////////////////////////////////////////////
// METHOD draw
//
// renders any graphics associated with this class (HUD, debug, etc)

void cViralChain::draw(
	c3DGameView *gview)
{
	sInt32 i;

	if (!mActive) return;

	// draw the chain
	for (i=0; i<mNumViralCodes; i++)
	{
		// render mesh
		gview->getDevice()->SetTransform(D3DTS_WORLD, &mTM);
		gAVRTContext.mGameState->mViralData->renderChainLink(
			1.0f,
			mViralCodes[i],
			mCurMeshVariant[i],
			this);

		// next slot in X
		mTM.m[3][0] += 1.0f;
	}
}

////////////////////////////////////////////////////////////////////////////////
// METHOD cViralChain
//
// constructor : nullify any class memory, reset to defaults

cViralChain::cViralChain(void)
{
	memset(this, 0, sizeof(cViralChain));
}

